! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!+
! Module: proton_mod
!
! Authors:  Brendan Perry - NASA/GSFC (brendan.h.perry@nasa.gov)
!           (BP) comprehensive rewrite and modularization for SAS-19
!           Steven L. Snowden - NASA/GSFC - original algorithm
!           
! 
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: proton_mod.f90,v 1.3 2009/02/26 16:39:45 aibarra Exp $
!-
!  Version 5.0  09 November 2012
!  Version 6.0/0.1  01 March 2020 (rewrite by BP, added SAS DAL/CAL calls)
!  Version 0.11 20 April 2020 (reconciliation b/w old esas version numbers)
!
!  Casts a map of the soft proton background using the fitted
!  spectral parameters for a power law (works) and broken power
!  law (in progress).  Soft proton detector maps developed by
!  K. Kuntz are used for the distributions. This new version
!  supercedes the proton within the old esas package. 
!
!  Usage:
!
!  OLD: proton objimage=1S001 ccd1=1 ccd2=1 ccd3=1 ccd4=1 ccd5=1 ccd6=1 ccd7=1 \
!       elow=750 ehigh=1250 spectrumcontrol=1 pindex=0.441324 pnorm=1.7351400E-02
!
!  NEW:
!
!     imagefile - Object Image (dataset)
!     specfile  - Object Spectral file (dataset)
!     withsky   - Perform det to sky transformation with esky2det/rotdet2sky?
!         spmapdet  - Output soft proton bkg map in det coords, defaults
!     spmapsky  - Output soft proton bkg map in sky coords, defaults
!     spectrol  - spectrum control (1=powerlaw,2=broken power law)
!     elow     - lower energy band limit in eV
!     ehigh    - upper energy band limit in eV
!     ccds     - CCDs/Quads to select
!     pindex   - power law index              (speccontrol=1) from Xspec
!     pnorm - power law normalization         (speccontrol=1) from Xspec
!     bindl - soft power law index            (speccontrol=2) from Xspec
!     break - power law break energy (in keV) (speccontrol=2) from Xspec
!     bindh - hard power law index            (speccontrol=2) from Xspec
!     bnorm - broken power law normalization  (speccontrol=2) from Xspec
!
module proton_mod

  use dal
  use cal
  use param
  use gofutils_mod
  use dataFilter
  use errorHandling
  use sasdal
  use gofio_mod
  

  implicit none

  private

  character(len=*), parameter :: &
       fovimgpar    = "imagefile", &     ! was: objfile from 'prefix'
       fovspecpar   = "specfile", &      ! was: spec_name
       !skypar       = "withsky", &       ! create sky coord output SP map?
       outdetpar    = "spmapdet", &      ! output SP bkg map in det coords
       !outskypar    = "spmapsky", &      ! output SP bkg map in sky coords
       spectrolpar  = "speccontrol", &   ! was: spectrumcontrol 1 or 2
       elowpar      = "elow", &          ! low energy bound eV 
       ehighpar     = "ehigh", &         ! high energy bound eV
       ccdspar      = "ccds", &          ! replaced ccd1, ccd2, etc
       pindexpar    = "pindex",&         ! power law index
       pnormpar     = "pnorm",&          ! power law normalization
       bindlpar     = "bindl",&          ! broken power law low index
       bindhpar     = "bindh",&          ! broken power law high index
       bbreakpar    = "bbreak",&         ! broken power law break energy
       bnormpar     = "bnorm"            ! broken power law normalization

  public proton

contains

  !---------------------------------------------------------------------
  subroutine proton
    !---------------------------------------------------------------------

    type(DataSetT) :: inFOVIMG_S, inFOVSPEC_S, outSPdetmap_S, outFOVIMG_S
    type(ArrayT) :: inFOVIMG_A, outSPdetmap_A, out_A
    type(TableT) :: inFOVSPEC_T, inFOVIMG_T
    !    type(ColumnT) :: channelS
    character (len=2) :: inst, instO, instS, cc, bandd, &
         bands(6) = (/ 'B1','B2','B3','B4','B5','B6' /)
    character (len=5) :: celow, cehigh, expid, expidO, expidS
    character (len=6) :: filtO, filtS, &
         filter(3) = (/ 'THIN  ','MEDIUM','THICK ' /) 
    character (len=10) :: prefix, obsid, obsidO, obsidS, ctype(2), cunit(2), radecsys
    character (len=12) :: str_crval1, str_crval2, str_detx, str_dety, estring
    character (len=18) :: type, suffix, logfile
    character (len=25) :: smode, smodeO, smodeS, submode
    character (len=80) :: line
    character (len=256) :: inFOVIMG_F, inFOVSPEC_F, outSPskymap_F, &
         outSPdetmap_F, outFOVIMG_F
    character (len=1024) :: command
    integer, dimension(2) :: outIMGdim
    integer :: chansobs, instnum, typeF, nRowsO, nRowsS, revolut, revolutO, &
         revolutS, spectrol, nChans, spXsize, spYsize, ff, &
         elowO, elowS, ehighO, ehighS, frmtime, crap, reason, &
         imgXsize, imgYsize, imgCounts
    integer :: elow, ehigh, iext, i, i1, i2, ib, ibb(7), ih, ios, ii, ii1, &
         ii2, iii, il, j, jj, k, kk, lun, nbands, n, nCCDs, typeA, l, m
    integer(kind=int16), dimension(:,:), pointer :: aDI16
    integer(kind=int32), dimension(:,:), pointer :: aDI32, inFOVIMG_JP
    ! check channels for mos AND pn ???!!!
    integer, parameter :: specchans = 2400, flareX = 780, flareY = 780 ! cal file
    integer, parameter :: xSize = 900, ySize = 900 ! hardcode forever???!!!
    integer :: status = 0
    logical :: doband(6), ccds(7) ! bhp: one array for all ccds/quads needed
    logical :: powerlaw, withsky, i_ok, warndie, die

    real(single), dimension(:,:), pointer :: outSPdetmap_RP, flaremap_RP, aDR32
    real(double), dimension(:,:), pointer :: aDR64
    real(single), dimension(:,:), pointer :: inFOVIMG_RP
    integer :: beginning, end, ratte
    real :: tempp, backscal, exposure
    real :: bindl, break, bindh, bnorm, bnormh, bnorml, &
         eh, el, exposen, pindex, pnorm, SPrate, rind, rn, &
         scale, sphb, splb, temp
    real(kind=double) :: crval(2), cdelt(2), crpix(2), crota(2), &
         dxcen, dycen, ra_pnt, dec_pnt, pa_pnt
    real :: eLo(6) = (/300.0, 750.0, 1250.0, 2000.0, 4000.0, 8000.0/) ! old elim
    real :: eHi(6) = (/749.0, 1249.0, 1999.0, 3999.0, 7999.0, 12000.0/)
    real :: backO, areaO, expoO, backS, areaS, expoS, scaleFactor(6)
    integer date_time1(8),date_time2(8)
    character(len=10) :: a(3), b(3)

    !
    backO = 0.
    areaO = 0.
    expoO = 0.
    backS = 0.
    areaS = 0.
    expoS = 0.
    powerlaw = .FALSE.
    ccds = .FALSE.
    !
    ! If the img and spec file are not good, may as well quit early! (BHP)
    ! Use this input image file as template, transferring relevant
    ! keywords to output swcx image (BHP)
    ! input spectral file for getting EXPOSURE:
    inFOVSPEC_F = stringParameter(fovspecpar) ! was from prefix
    ! open and read input image file inFOVIMG_F
    status = 0
    warndie = .TRUE. ! croak if not found or unreadable
    call check_open_set(inFOVSPEC_F,inFOVSPEC_S,1,status,warndie) ! dgoflib/gofio_mod
    warndie = .TRUE.     ! see read_header_info in gofio_mod for behavior
    typeF = 2 ! 1 = evli, 2 = spec, 3 = image
    call read_header_info(typeF,inFOVSPEC_S,inFOVSPEC_T,out_A,instS,obsidS,&
         expidS,smodeS,frmtime,filtS,revolutS,backS,areaS,expoS,elowS,&
         ehighS,nRowsS,warndie)
    backscal = backS/1440000.0
    !
    inFOVIMG_F = stringParameter(fovimgpar) ! was from prefix
    ! open and read input image file inFOVIMG_F a 780x780 Int32 image array
    ! OLD/NEW: mos1S001-obj-im-det-2000-7200.fits/mos1S001-fovimdet-2000-7200.fits
    status = 0
    warndie = .TRUE. ! croak if not found or unreadable
    call check_open_set(inFOVIMG_F,inFOVIMG_S,1,status,warndie) !goflib/gofio_mod
    warndie = .TRUE.     ! see read_header_info in gofio_mod for behavior
    typeF = 3 ! 1 = evli, 2 = spec, 3 = image
    call read_header_info(typeF,inFOVIMG_S,inFOVIMG_T,out_A,instO,obsidO,&
         expidO,smodeO,frmtime,filtO,revolutO,backO,areaO,expoO,elowO,&
         ehighO,nRowsO,warndie)
    if (instO .eq. 'M1') then
       prefix = 'mos1' // trim(adjustl(expidO))
    elseif (instO .eq. 'M2') then
       prefix = 'mos2' // trim(adjustl(expidO))
    elseif (instO .eq. 'PN') then
       prefix = 'pn' // trim(adjustl(expidO))
    else
       call error ("noINST","Instrument from image must be M1/M2/PN")
    endif
    !
    call read_2Darray(inFOVIMG_S,inFOVIMG_A,aDR32,aDR64,aDI16,inFOVIMG_JP,&
         imgXsize,imgYsize,typeA)
    !
    call system_clock(beginning, ratte)
    call CAL_setState(inFOVIMG_F) ! for later CAL usage for CCF extraction
    call system_clock(end)
    write(errstr,*)"First CAL_setState call elapsed time: ", real(end - beginning) / real(ratte)
    call message(AppMsg,Verbose) 
    !
    outIMGdim(1) = imgXsize
    outIMGdim(2) = imgYsize
    inFOVIMG_JP => int32Array2Data(inFOVIMG_A) ! FOV image array
    imgCounts = sum(inFOVIMG_JP)
    !
    nCCDs = 7      ! MOS chips
    nChans = 2400  ! MOS spectral channels
    if (instO .eq. 'PN') then 
       nCCDs = 4
       nChans = 4096
    end if
    do i = 1, nCCDs
       ccds(i) = booleanParameter(ccdspar,i-1)
    end do
    if ( (instO .eq. 'M1') .and. revolutO .gt. 961) then
       if (ccds(6)) then
          call warning ("PostCCD6Loss",&
               "CCD#6 was selected but orbit after 961, ccds(6) set to F")
          ccds(6) = .FALSE.
       end if
    end if
    elow = intParameter(elowpar)
    write(celow,"(I5)") elow
    ehigh = intParameter(ehighpar)
    write(cehigh,"(I5)") ehigh
    estring = "-" // trim(adjustl(celow)) // "-" // trim(adjustl(cehigh))
    if (elow .ge. ehigh) then
       call error("elowGEehigh","Parameter elow must be less than ehigh")
    end if
    !
    !  Create output PROT detector map filename(s)
    !
    ! OLDNEW: mos1S001-prot-im-det-300-1250.fits/mos1S001-protimdet-300-1250.fits
    ! OLDNEW: mos1S001-prot-im-sky-300-1250.fits/mos1S001-protimsky-300-1250.fits
    ! what to do if no elow ehigh and/or default???!!
    ! user-specified, but if still 'default', generate filename
    outSPdetmap_F = stringParameter(outdetpar)
    if (trim(adjustl(outSPdetmap_F)) .eq. 'default') then
       outSPdetmap_F = trim(adjustl(prefix)) // "-protimdet" // &
            trim(adjustl(estring)) // ".fits"
    end if
    die = .FALSE.
    call check_clobber(outSPdetmap_F,die)
    ! BHP: deprecated for now 
    !    withsky = booleanParameter(skypar)
    !    if (withsky) then
    !       outSPskymap_F = stringParameter(outskypar)
    !       if (trim(adjustl(outSPskymap_F)) .eq. 'default') then
    !          outSPskymap_F = trim(adjustl(prefix)) // "-protimsky" // &
    !               trim(adjustl(estring)) // ".fits"
    !       end if
    !       die = .FALSE.
    !       call check_clobber(outSPskymap_F,die)
    !    endif
    ! 
    spectrol = intParameter(spectrolpar)
    if (spectrol .eq. 1) powerlaw = .TRUE.
    !
    if (powerlaw) then 
       pindex = realParameter(pindexpar)
       pnorm  = realParameter(pnormpar)
    else    
       bindl  = realParameter(bindlpar)
       bindh  = realParameter(bindhpar)
       break  = realParameter(bbreakpar)
       bnorm  = realParameter(bnormpar)
    endif
    !
    ! SP map fits image is a clone of the FOV DET image insofar as header keywords.
    ! check this???!!! add more/remove some keywords
    !
    ! sanity check
    if ((trim(instS) .ne. trim(instO)) .or. (trim(obsidS) .ne. trim(obsidO)) &
         .or. (trim(expidS) .ne. trim(expidO)) .or. (revolutS .ne. revolutO)) then
       call error("IMG.ne.SPEC",&
            "Image and Spectrum have different OBSID/EXPID/INST/ORBIT")
    endif
    !
    ! print out starting values
    !
    call printParams (nCCDs,inFOVIMG_F,inFOVSPEC_F,outSPdetmap_F,celow,cehigh,&
         powerlaw,ccds,pindex,pnorm,bindl,bindh,break,bnorm)
    !
    ! Make sure that the line energies are in keV 
    ! input image array is bitpix = 32, 32-bit signed INTEGER 
    !  output array image bitpix = -32, 32-bit FLOAT
    ! open output image file outSPskymap_F
    status = 0
    warndie = .TRUE. ! croak if not found or unreadable
    call check_open_set(outSPdetmap_F,outSPdetmap_S,3,status,warndie)
    ! 3 = CREATE, in gofio
    ! add the real32 SP map to output file
    outSPdetmap_A = addArray(outSPdetmap_S,"PRIMARY",real32,outIMGdim)
    outSPdetmap_RP => real32Array2Data(outSPdetmap_A)
    outSPdetmap_RP = 0. ! zero it out
    !
    ! clean these keywords up and add HISTORY/COMMENT lines???!!!
    call copyAllAttributes(attributable(inFOVIMG_A),attributable(outSPdetmap_A))
    call copyAllAttributes(attributable(inFOVIMG_S),attributable(outSPdetmap_S))
    call release(inFOVIMG_S) ! input image no longer needed

    write(errstr,*) "FOV exposure (from spectrum): ", expoS
    call message(AppMsg,Always)
    write(errstr,*) "Input image total counts    : ", imgCounts
    call message(AppMsg,Always)
    !
    !  Set the bands to be included
    ! Loop through the selected SP bands
    ! eLo(6) = (/300.0, 750.0, 1250.0, 2000.0, 4000.0, 8000.0/)
    ! eHi(6) = (/749.0, 1249.0, 1999.0, 3999.0, 7999.0, 12000.0/)
    !
    doband = .FALSE.
    scaleFactor = 1.
    j = 0
    do i = 1, 6
       if ( elow .gt. eHi(i) .and. ehigh .gt. eHi(i)) then
          ! both elow and ehigh above band range, just fill in til band selected reached
          doband(i) = .false. 
          scaleFactor(i) = 1.
          cycle
       endif
       if ( elow .ge. eLo(i) .and. ehigh .le. eHi(i) ) then ! both within single band
          doband(i) = .true.
          doband(i+1:6) = .false.
          scaleFactor(i) = (ehigh - elow) / (eHi(i) - eLo(i))
          scaleFactor(i+1:6) = 1.
          exit
       end if
       if ( elow .le. eLo(i) .and. ehigh .lt. eLo(i)) then
          ! both elow and ehigh below band range, fill in rest of bands and exit
          doband(i:6) = .false. 
          scaleFactor(i:6) = 1.
          exit
       endif
       ! all other possibilities, step thru
       do j=i,6
             if (ehigh .gt. eHi(j)) then
                doband(j) = .true.
                scaleFactor(j) = (eHi(j) - elow) / (eHi(j) - eLo(j))
                cycle
             else
                doband(j) = .true.
                scaleFactor(j) = (ehigh - eLo(j)) / (eHi(j) - eLo(j))
                exit                
             endif          
       end do
    end do
    !
    write(errstr,*) 'Bands and scale factors for the SP maps:'
    call message(AppMsg,Always)
    do i = 1, 6
       write(cc,"(I1)")i
       write(errstr,*) "Band(",trim(cc),") = ",doband(i),&
            " SF(",trim(cc),") = ",scaleFactor(i)
       call message(AppMsg,Always)
    end do
    nbands = count(doband)    
    !
    ! Loop through the selected SP bands
    ! bands are B1/B2/B3/B4/B5/B6
    ! energies are .7/.1/.2/.3/.4/.5/.6 
    ! filters are THIN/MEDIUM/THICK
    !
    do j=1, nbands ! number of bands selected
       do ff=1,3   ! filters THIN1/MEDIUM/THICK
          call system_clock(beginning, ratte)
          select case (ff)
          case (1)
             write(errstr,*) 'filter = Thin1','  band = ',bands(j)
             call message(AppMsg,Always)
             call CAL_setState(filterId=THIN1)
          case (2)
             write(errstr,*) 'filter = Medium',' band = ',bands(j)
             call message(AppMsg,Always)
             call CAL_setState(filterId=MEDIUM)
          case (3)
             write(errstr,*) 'filter = Thick','  band = ',bands(j)
             call message(AppMsg,Always)
             call CAL_setState(filterId=THICK)
          end select
          call system_clock(end)
          write(errstr,*)ff," Second CAL_setState call elapsed time: ", &
               real(end - beginning) / real(ratte)
          call message(AppMsg,Verbose) ! change to Verbose when sorted???!!!
          !
          ! BHP: trying to get to the bottom of this taking 10 mins sometimes???!!!
          !
          call system_clock(beginning, ratte)
          call CAL_flaremap(bands(j),flaremap_RP)
          call system_clock(end)
          write(errstr,*)"CAL_flaremap call elapsed time: ", &
               real(end - beginning) / real(ratte)
          call message(AppMsg,Verbose) ! change to Verbose when sorted???!!!
          !
          !  Co-add the data from all filters
          !
          do k=0,flareX-1     ! 780
             do kk=0,flareY-1 ! 780
                outSPdetmap_RP(k,kk) = outSPdetmap_RP(k,kk) + &
                     scaleFactor(j)*flaremap_RP(k,kk)
             enddo
          enddo
          !
          ! BHP: Mask chips if necessary (zero out chips not selected by user).
          ! The long code that was below has been moved to a subroutine in
          ! gofutils_mod.f90 in the task goflib.
          !
          write(errstr,*) "Sum of outSPdetmap Before maskchips = ",sum(outSPdetmap_RP)
          call message(AppMsg,Always) ! change to Verbose???!!!
          do i=1,nccds
             if (.not.ccds(i)) then
                write(errstr,*) "Calling maskchips for CCD/QUAD = ",i,&
                     ' inst = ',instS
                call message(AppMsg,Verbose)
                call maskchips(instS,i,outSPdetmap_RP)
             end if
          end do
          write(errstr,*) "Sum of outSPdetmap After maskchips = ",sum(outSPdetmap_RP)
          call message(AppMsg,Always) ! change to Verbose???!!!
       enddo
    enddo
    !
    !  SS: Find the number of counts in the detector map
    !
    rn = 0.
    rn = sum(outSPdetmap_RP)
    !
    !  SS: Determine the number of SP counts in the band
    !
    eh = ehigh/1000.0
    el = elow/1000.0
    if(powerlaw) then
       rind = 1.0 - pindex
       SPrate = pnorm*(eh**rind - el**rind)/rind
    else
       splb = break**(-bindl)
       sphb = break**(-bindh)
       if(break .lt. 1.0) then
          bnorml = bnorm*sphb/splb
          bnormh = bnorm
       elseif (break .eq. 1.0) then
          bnorml = bnorm
          bnormh = bnorm
       else
          bnorml = bnorm
          bnormh = bnorm*splb/sphb
       endif
       if(eh .le. break) then
          rind = 1.0 - bindl
          SPrate = bnorm*(eh**rind - el**rind)/rind
       elseif(el .ge. break) then
          rind = 1.0 - bindh
          SPrate = bnorm*(break**(bindh-bindl))* &
               (eh**rind - el**rind)/rind
       else
          rind = 1.0 - bindl
          temp = bnorm*(break**rind - el**rind)/rind
          rind = 1.0 - bindh
          SPrate = temp + bnorm*(break**(bindh-bindl))* &
               (eh**rind - break**rind)/rind
       endif
    endif
    write(errstr,*) 'Soft Proton Event Rate: ', SPrate ! OLDNEW: prot/SPrate
    call message(AppMsg,Always)
    !
    !  SS: Set the scale factor
    !
    SPrate = expoS * SPrate
    scale = SPrate / rn
    ! Multiply all elements by the scale factor
    do i=1,flareX
       do j=1,flareY
          outSPdetmap_RP(i-1,j-1) = scale * outSPdetmap_RP(i-1,j-1)
       end do
    end do
    !
    write(errstr,*) "Observation exposure  : ", expoS
    call message(AppMsg,Always)
    write(errstr,*) "Detector map counts   : ", rn
    call message(AppMsg,Always)
    write(errstr,*) "Proton flaring events : ", SPrate
    call message(AppMsg,Always)
    write(errstr,*) "Scale factor          : ", scale
    call message(AppMsg,Always)
    call release(outSPdetmap_S)
!    !
!    rn = 0.
!    rn = sum(outSPdetmap_RP)
!    write(errstr,2140) rn
!    call message(AppMsg,Always)
!2140 format(1x,'Total PROT counts in band: ',f14.2)
!    ! add more history/comment keywords???
!    call addlohi(outSPdetmap_F,elow,ehigh) ! add ELOW, EHIGH keywords yes or no???!!!
! This may be added in SAS-22???!!!
!    if (withsky) then
!       warndie = .FALSE.
!       call coord_keys (outSPdetmap_S,ra_pnt,dec_pnt,pa_pnt,crval,cdelt, &
!            cunit,ctype,crpix,crota,radecsys,warndie)
!    endif
!    call release(outSPdetmap_S) ! close output SWCX Map fits file
!
!    !
!    ! get coordinate keywords from header to run rotimdet2sky
!    !
!    warndie = .true.
!    call coord_keys (outSPdetmap_S,ra_pnt,dec_pnt,pa_pnt,crval,cdelt, &
!         cunit,ctype,crpix,crota,radecsys,warndie)
!    call release(outSPdetmap_S)
!    logfile = "temppp.out"
!    write(str_crval1,'(F12.5)')ra_pnt
!    write(str_crval2,'(F12.5)')dec_pnt
!    command = "esky2det datastyle=user " // "ra=" // trim(adjustl(str_crval1)) // &
!         " dec=" // trim(adjustl(str_crval2)) // " outunit=det withheader=no" // &
!         " calinfostyle=set calinfoset=" // trim(adjustl(outSPdetmap_F)) // &
!         " verbosity=0 " // '>& ' // trim(adjustl(logfile))
!    write(errstr,*) trim(adjustl(command))
!    call message(AppMsg,Always)
!    call system(trim(adjustl(command)))
!    !
!    i_ok = .false.
!    ios = 0
!    inquire(FILE=trim(logfile), EXIST=i_ok)
!    if (i_ok) then
!       write(errstr,*) "Ran esky2det, opening and reading dxcen/dycen from log: ", &
!            trim(adjustl(logfile))
!       call message(AppMsg,Noisy)
!       lun = GetLun() ! in goflib/gofutils
!       ios = 0
!       open (lun,file=trim(logfile), iostat=ios)
!       if (ios .ne. 0) then
!          call error ("noLUN","Cannot get LUN for esky2det text output file")
!       end if
!       do ! esky2det dumps crap into temppp.out text file ???!!!
!          read(lun,'(A)',iostat=reason) line
!          if (reason > 0)  then
!             call error ("badTEXTread","Error reading output temppp.out esky2det file")
!          else if (reason < 0) then ! EOF
!             exit
!          else
!             crap = 0
!             crap = scan(line,'abcdefghijklmnopqrstuvwxyz')
!             if (crap .gt. 0) then
!                cycle
!             else
!                read(line,*)dxcen,dycen
!                exit
!             end if
!          end if
!       end do
!       write(errstr,*) "DXCEN/DYCEN from esky2det: ",dxcen,dycen
!       call message(AppMsg,Always)
!       call unlink ("temppp.out") ! remove temp file
!    else
!       write(errstr,*)"esky2det temp output file (temppp.out) cannot be read "
!       call error("badEsky2DetRun",trim(errstr))
!    end if
!    !
!    if (withsky) then
!       write(str_detx,'(F12.5)')dxcen
!       write(str_dety,'(F12.5)')dycen
!       command = "rotdet2sky intemplate=" // trim(adjustl(inFOVIMG_F)) // &
!            " inimage=" // trim(adjustl(outSPdetmap_F)) // &
!            " outimage=" // trim(adjustl(outSPskymap_F)) // &
!            " withmask=F " // & 
!            " withskyxy=T" // &
!            " detx=" // trim(adjustl(str_detx)) // &
!            " dety=" // trim(adjustl(str_dety)) // &
!            " skyx=" // trim(adjustl(str_crval1)) // & 
!            " skyy=" // trim(adjustl(str_crval2))
!       write(errstr,*) trim(adjustl(command))
!       call message(AppMsg,Always)
!       print*,trim(adjustl(command))
!       call system(trim(adjustl(command)))
!       call addlohi(outSPskymap_F,elow,ehigh)
!    end if
!    ! add history/comment keywords???
!    call addlohi(outSPdetmap_F,elow,ehigh)
!    ! close the new SP Map fits file    
!    
!    !call release(outSPdetmap_S)
    
    
    return

  end subroutine proton

  subroutine printParams (nCCDs,inIMG_F,inSPEC_F,outSPdetmap_F,celow,cehigh,&
       powerlaw,ccds,pindex,pnorm,bindl,bindh,break,bnorm)

    character(len=*), intent(in) :: inIMG_F, inSPEC_F, outSPdetmap_F, celow, cehigh
    character(len=5) :: foo
    character(len=1) :: cc
    integer :: i, nCCDs
    logical, intent(in) :: ccds(7), powerlaw
    real, intent(in) :: pindex, pnorm, bindl, bindh, break, bnorm
    
    write(errstr,*) "Input parameters:"
    call message(AppMsg,Always)
    write(errstr,*) "Input Spectral Filename     : ",trim((adjustl(inSPEC_F)))
    call message(AppMsg,Always)
    write(errstr,*) "Input Image Filename        : ",trim(adjustl(inIMG_F))
    call message(AppMsg,Always)
    write(errstr,*) "Output SP Map (det) Filename: ",trim(adjustl(outSPdetmap_F))
    call message(AppMsg,Always)
!    if (withsky) then 
!       write(errstr,*) "Output SP Map (sky) Filename: ",trim(adjustl(outSPskymap_F))
!       call message(AppMsg,Always)
!    endif
    write(errstr,*) "Selected energy lower limit : ",trim(adjustl(celow))
    call message(AppMsg,Always)
    write(errstr,*) "Selected energy upper limit : ",trim(adjustl(cehigh))
    call message(AppMsg,Always)
    if (powerlaw) then
       write(errstr,*) "Spectral control is power law: "
       call message(AppMsg,Always)
       write(errstr,*) "Photon index             : ",pindex
       call message(AppMsg,Always)
       write(errstr,*) "Photon normalization     : ",pnorm
       call message(AppMsg,Always)
    else 
       write(errstr,*) "Spectral control is broken power law: "
       call message(AppMsg,Always)
       write(errstr,*) "Broken Power law low-energy index   : ",bindl
       call message(AppMsg,Always)
       write(errstr,*) "Broken Power law break energy (keV) : ",break
       call message(AppMsg,Always)
       write(errstr,*) "Broken Power law high-energy index  : ",bindh
       call message(AppMsg,Always)
       write(errstr,*) "Broken Power law normalization      : ",bnorm
       call message(AppMsg,Always)
    end if
    foo = "CCD("
    if (nCCDS .eq. 4) foo = "QUAD("
    do i = 1, nCCDs
       write(cc,"(I1)")i
       write(errstr,*) trim(adjustl(foo)),cc,") selected? (T/F)",ccds(i)
       call message(AppMsg,Always)
    end do

    return
  end subroutine printParams

end module proton_mod
